#
# oops-cert function api
#

get_randfile() {
	randfiles=''
	for file in /var/log/messages /var/adm/messages \
		/kernel /vmunix /boot/vmlinuz \
		/etc/hosts /etc/resolv.conf ~/.rand; do
		if [ -f $file ]; then
			if [ ".$randfiles" = . ]; then
				randfiles="$file"
			else
				randfiles="${randfiles}:$file"
			fi
		fi
	done

	retvalue="RETVAL=\"${randfiles}\""
	eval ${retvalue}
}

make_keyfile() {
	_openssl=$1
	_destdir=$2
	_keyname=$3

	[ -f "${_destdir}/${_keyname}.key" ] && return 0

	[ ! -f $HOME/.rnd ] && touch $HOME/.rnd
	get_randfile
	if [ ".${RETVAL}" != "." ]; then
	    ${_openssl} genrsa -rand ${RETVAL} \
						-out ${_destdir}/${_keyname}.key \
						2048
	else
		${_openssl} genrsa -out ${_destdir}/${_keyname}.key 2048
	fi

	return $?
}

request() {
	_msg="$1"
	_def="$2"

	RETVAL=
	RET_ANSWER=
	while [ -z "${RET_ANSWER}" ]
	do
		echo -n "${_msg}"
		read RET_ANSWER

		[ -n "${_def}" -a -z "${RET_ANSWER}" ] && RET_ANSWER="${_def}"
		[ "${_def}" = "-" -a -z "${RET_ANSWER}" ] && break
	done

	eval "RETVAL=\"${RET_ANSWER}\""
}

make_conf() {
	_type=$1
	_conf=$2

	[ "${_type}" != "server" -a "${_type}" != "ca" ] && _type="server"

	C="KR"
    S=""
    L="Seoul"
    [ "$_type" = "ca" ] && O="Test Cert Agency" || O="Example Compnay"
    [ "$_type" = "ca" ] && OU="Self Issued Certificate Authority" || OU="CA Team"
    [ "$_type" = "ca" ] && CN="Self CA" || CN="localhost"
    E="root@localhost"

	request $"    1. Country Name           (2 letter code) [${C}]: " "$C"
	req_DN="countryName                     = \"1. Country Name\"\n"
	req_DN="${req_DN}countryName_min                 = 2\n"
	req_DN="${req_DN}countryName_max                 = 2\n"
	req_DN="${req_DN}countryName_default             = ${RETVAL}\n"

	request $"    2. Organization Name      (eg, company)   [${O}]: " "$O"
	req_DN="${req_DN}0.organizationName              = \"2. Organization Name\"\n"
	req_DN="${req_DN}0.organizationName_default      = ${RETVAL}\n"
	request $"    3. Organization Unit Name (eg, section)   [${OU}]: " "$OU"
	req_DN="${req_DN}organizationalUnitName          = \"3. Organization Unit Name\"\n"
	req_DN="${req_DN}organizationalUnitName_default  = ${RETVAL}\n"

	msg=$"    4. Common Name            (eg,"
    [ "${_type}" = "ca" ] && msg="${msg} CA Name)   [${CN}]: " || msg="${msg} FQDN)      [${CN}]: "
	request "${msg}" "$CN"
	req_DN="${req_DN}commonName                      = \"4. Common Name\"\n"
	req_DN="${req_DN}commonName_default              = ${RETVAL}\n"

	if [ "${_type}" = "server" ]; then
		ANSWER=
		while [ -z "${ANSWER}" ]
		do
			echo -n $"       Subejct Alternative Name (FQDN)               : "
			read ANSWER

			[ -z "${ANSWER}" ] && break;
			[ -z "${SANAMES}" ] && SANAMES="DNS:${RETVAL},DNS:${ANSWER}" || \
									SANAMES="${SANAMES},DSN:${ANSWER}"
		done
	fi

	request $"    5. Email Address          (eg, id@FQDN)   [${E}]: " "$E"
	req_DN="${req_DN}emailAddress                    = \"5. Email Address\"\n"
	req_DN="${req_DN}emailAddress_default            = ${RETVAL}\n"

	cat <<EOF > $_conf
# -------------- BEGIN custom openssl.cnf -----
[ req ]
default_bits                    = 2048
distinguished_name              = req_DN
#attributes                     = req_attributes
EOF

if [ "${_type}" = "server" ]; then
	[ -n "${SANAMES}" ] && echo "req_extensions                  = v3_req" >> $_conf
	echo "extensions                      = server" >> $_conf
else
	echo "extensions                      = ca_ext" >> $_conf
fi

	cat <<EOF >> $_conf

[ req_DN ]
EOF

echo -e ${req_DN} >> $_conf

	cat <<EOF >> $_conf

[ ca_ext ]
basicConstraints                = CA:true,pathlen:0
nsComment                       = "OOPS.org generated custom CA certificate"
nsCertType                      = sslCA, emailCA
subjectKeyIdentifier            = hash
authorityKeyIdentifier          = keyid:always,issuer:always
subjectAltName                  = email:copy

[ server ]
basicConstraints                = CA:FALSE
nsComment                       = "OOPS.org generated custom server certificate"
nsCertType                      = server
subjectKeyIdentifier            = hash
authorityKeyIdentifier          = keyid,issuer:always

[ v3_req ]
basicConstraints                = CA:FALSE
subjectAltName                  = ${SANAMES}
EOF
}

make_phrase() {
	_openssl=$1
	_keyfile=$2

	${_openssl} rsa -des3 \
					-in ${_keyfile} \
					-out ${_keyfile}.crypt

	res=$?
	[ $res -eq 0 ] && mv -f ${_keyfile}.crypt ${_keyfile}

	return $res
}

selfsign() {
	_expire=$1
	_openssl=$2
	_path="$3"

	$_openssl x509 -extfile ${_path}.cnf \
					-days $_expire \
					-signkey ${_path}.key \
					-in ${_path}.csr \
					-req \
					-out ${_path}.crt
	return $?
}

verify() {
	_openssl=$1
	_path="$2"
	_cafile="$3"

	[ -n "${_cafile}" -a -f "${_cafile}" ] && opt="-CAfile ${_cafile}" || opt=

	${_openssl} verify ${opt} ${_path}.crt 2> /dev/null | grep 'OK' >& /dev/null
	return $?
}

make_sign_conf() {
	_conf=$1

	cat <<EOF > $_conf
[ ca ]
default_ca              = CA_default

[ CA_default ]
dir                     = /usr/share/ssl/build/data
certs                   = \$dir
new_certs_dir           = \$dir/ca.db.certs
database                = \$dir/ca.db.index
serial                  = \$dir/ca.db.serial
RANDFILE                = \$dir/ca.db.rand
certificate             = \$dir/ca.crt
private_key             = \$dir/ca.key
default_days            = 3652
default_crl_days        = 30
default_md              = md5
preserve                = no
policy                  = policy_oops
copy_extensions         = copy
#x509_extensions        = usr_cert

[ policy_oops ]
countryName             = optional
stateOrProvinceName     = optional
localityName            = optional
organizationName        = optional
organizationalUnitName  = optional
commonName              = supplied
emailAddress            = optional

[ usr_cert ]
basicConstraints        = CA:FALSE
nsComment               = "OOPS.org Signing Generator"
subjectKeyIdentifier    = hash
authorityKeyIdentifier  = keyid,issuer:always
EOF
}

# Local variables:
# tab-width: 4
# c-basic-offset: 4
# End:
# vim: set filetype=sh noet sw=4 ts=4 fdm=marker:
# vim<600: noet sw=4 ts=4:
